var FdThumbnail = {
  RATIO: 0.75,

  getHeight: function(width) {
    return Math.floor(width * FdThumbnail.RATIO);
  },

  getWidth: function(height) {
    return Math.floor(height / FdThumbnail.RATIO);
  },

  createBase64Thumbnail: function(wnd, thumbWidth) {
    var canvas = document.createElementNS("http://www.w3.org/1999/xhtml", "canvas");
    canvas.width = thumbWidth;
    canvas.height = FdThumbnail.getHeight(thumbWidth);

    var context = canvas.getContext("2d");
    context.clearRect(0, 0, canvas.width, canvas.height);

    var width = FdThumbnail.getWindowWidth(wnd);
    var height = FdThumbnail.getHeight(width);
    context.scale(canvas.width / width, canvas.height / height);
    context.drawWindow(wnd, 0, 0, width, height, "white");

    return canvas.toDataURL("image/png")
           .replace(/^data:image\/png;base64,/, "");
  },

  getWindowWidth: function(wnd) {
    return wnd.document.width || wnd.innerWidth;
  },

  createCanvasThumbnail: function(wnd, thumbWidth) {
    return atob(FdThumbnail.createBase64Thumbnail(wnd, thumbWidth));
  },

  // Create full image using javascript, then scale the image using java
  // Note: couldn't find a way to pass binary strings to/from Java code,
  // so passed base64 encoded strings instead

  createJavaThumbnail: function(wnd, thumbWidth) {
    var data = FdThumbnail.createBase64Thumbnail(wnd, FdThumbnail.getWindowWidth(wnd));
    var bytes = new sun.misc.BASE64Decoder().decodeBuffer(data);

    var byteIn = new java.io.ByteArrayInputStream(bytes);
    var image = Packages.javax.imageio.ImageIO.read(byteIn);

    var thumbHeight = FdThumbnail.getHeight(thumbWidth);
    var tempImage = image.getScaledInstance(thumbWidth, thumbHeight, image.SCALE_SMOOTH);
    var result = new java.awt.image.BufferedImage(thumbWidth, thumbHeight, image.TYPE_INT_RGB);
    result.getGraphics().drawImage(tempImage, 0, 0, null);
    
    var byteOut = new java.io.ByteArrayOutputStream();
    Packages.javax.imageio.ImageIO.write(result, "png", byteOut);
    data = new sun.misc.BASE64Encoder().encode(byteOut.toByteArray());
    return atob(("" + data).replace(/\r\n/g, ""));
  },

  createThumbnail: function(wnd, thumbWidth) {
    if (FdUtils.isJavaEnabled() && FdPrefs.getBoolPref("useJava")) {
      return FdThumbnail.createJavaThumbnail(wnd, thumbWidth);
    }
    return FdThumbnail.createCanvasThumbnail(wnd, thumbWidth);
  },
}
